@@ -4,14 +4,17 @@ use 5.008_001;
 use Moose;
 use namespace::clean -except => ['meta'];
 
-BEGIN { extends 'Catalyst::Controller' }
+BEGIN { extends 'Catalyst::Controller::ActionRole' }
 
-our $VERSION = '0.07';
+our $VERSION = '0.08';
+
+__PACKAGE__->config(
+    action_roles => [ '+Catalyst::Controller::Resources::ActionRole::ResourceAction' ],
+);
 
 with qw(
     Catalyst::Controller::Resources::Role::BuildActions
     Catalyst::Controller::Resources::Role::ParseAttributes
-    Catalyst::Controller::Resources::Role::ActionRole
 );
 
 has '+_default_collection_actions' => (
@@ -51,37 +54,37 @@ Catalyst::Controller::Resources - Catalyst Collection Resources Controller
 
   package MyApp::Controller::Articles;
   use base 'Catalyst::Controller::Resources';
-  
+
   # GET /articles
   sub list {
       my ($self, $c) = @_;
   }
-  
+
   # POST /articles
   sub create {
       my ($self, $c) = @_;
   }
-  
+
   # GET /articles/{article_id}
   sub show {
       my ($self, $c, $article_id) = @_;
   }
-  
+
   # PUT /articles/{article_id}
   sub update {
       my ($self, $c, $article_id) = @_;
   }
-  
+
   # DELETE /articles/{article_id}
   sub destroy {
       my ($self, $c, $article_id) = @_;
   }
-  
+
   # GET /articles/new
   sub post {
       my ($self, $c) = @_;
   }
-  
+
   # GET /articles/{article_id}/edit
   sub edit {
       my ($self, $c, $article_id) = @_;
@@ -91,44 +94,44 @@ Catalyst::Controller::Resources - Catalyst Collection Resources Controller
 
   package MyApp::Controller::Articles;
   use base 'Catalyst::Controller::Resources';
-  
+
   # ...
-  
+
   package MyApp::Controller::Comments;
   use base 'Catalyst::Controller::Resources';
-  
+
   __PACKAGE__->config(belongs_to => 'Articles');
-  
+
   # GET /articles/{article_id}/comments
   sub list {
       my ($self, $c, $article_id) = @_;
   }
-  
+
   # POST /articles/{article_id}/comments
   sub create {
       my ($self, $c, $article_id) = @_;
   }
-  
+
   # GET /articles/{article_id}/comments/{comment_id}
   sub show {
       my ($self, $c, $article_id, $comment_id) = @_;
   }
-  
+
   # PUT /articles/{article_id}/comments/{comment_id}
   sub update {
       my ($self, $c, $article_id, $comment_id) = @_;
   }
-  
+
   # DELETE /articles/{article_id}/comments/{comment_id}
   sub destroy {
       my ($self, $c, $article_id, $comment_id) = @_;
   }
-  
+
   # GET /articles/{article_id}/comments/new
   sub post {
       my ($self, $c, $article_id) = @_;
   }
-  
+
   # GET /articles/{article_id}/comments/{comment_id}/edit
   sub edit {
       my ($self, $c, $article_id, $comment_id) = @_;
@@ -145,13 +148,13 @@ In your controller:
       Catalyst::Controller::Resources
       Catalyst::Controller::RequestToken
   );
-  
+
   sub post :CreateToken {
       my ($self, $c) = @_;
       $c->stash->{template} = 'foo/post.tt';
       $c->forward($c->view('TT'));
   }
-  
+
   sub create :ValidateToken {
       my ($self, $c) = @_;
 
